﻿cPet = {
	isActive = false,
	hasBeenActive = false,
	name = nil,
	level = nil,
	xp = nil,
	maxXP = nil,
	maxLevel = nil,
	killList = nil,
	killAverage = nil,
	killListLength = 100,
	
	nextMobName = nil, -- The name of the next mob to be announced... Dirty workaround :/
	
	Initialize = function(self)
		-- Make sure the player has a pet
		local hasUI, isHunterPet = HasPetUI();
		if not hasUI or not isHunterPet then
			self.isActive = false;
		else
			-- Update the object with the data for the current pet
			self:Update()
			self.killAverage = nil
			
			if not sData.pet.killList[self.name] then
				sData.pet.killList[self.name] = { }
			end
		end
	end,
	
	-- Updates the pet xp info
	Update = function(self)
		local oldXP, gainedXP
		oldXP = self.xp
		
		if UnitName("pet") == nil then
			return;
		end
	
		self.name = UnitName("pet")
		self.level = UnitLevel('pet')
		self.maxLevel = cPlayer.level or UnitLevel('player')
		self.xp, self.maxXP = GetPetExperience()
		
		if self.level < self.maxLevel then
			self.isActive = true;
			self.hasBeenActive = true;
		else
			self.isActive = false;
			self.hasBeenActive = false;
		end
		
		if oldXP then
			gainedXP = self.xp - oldXP
			-- Make sure this falls within realistic gains from a kill.
			-- Otherwise this may be an initialization update.
			if gainedXP < (PetXP(cPlayer.level, self.level, cPlayer.level) * 3) then
				self:AddKill(gainedXP)
			end
		end
		
		return gainedXP or 0
	end,
	
	AddKill = function(self, xp)
		if not sData.pet.killList then
			sData.pet.killList = { }
		end
		if not sData.pet.killList[self.name] then
			sData.pet.killList[self.name] = {}
		end
		if xp > 0 then
			self.killAverage = nil
			table.insert(sData.pet.killList[self.name], 1, xp)
			if(# sData.pet.killList[self.name] > self.killListLength) then
				table.remove(sData.pet.killList[self.name])
			end
		end
	end,
	
	GetAverageKillXP = function(self)
		if self.killAverage == nil then
			if sData.pet.killList[self.name] and (# sData.pet.killList[self.name] > 0) then
				local total = 0
				local maxUsed = # sData.pet.killList[self.name]
				if maxUsed > sConfig.averageDisplay.petKillListLength then
					maxUsed = sConfig.averageDisplay.petKillListLength
				end
				for index, value in ipairs(sData.pet.killList[self.name]) do
					if index > maxUsed then
						break;
					end
					total = total + (value or 0)
				end
				if total > 0 then
					self.killAverage = (total / maxUsed);
				else
					self.killAverage = PetXP(cPlayer.level, self.level, cPlayer.level)
					sData.pet.killList[self.name] = { }
					table.insert(sData.pet.killList[self.name], tonumber(self.killAverage))
				end
			else
				self.killAverage = PetXP(cPlayer.level, self.level, cPlayer.level)
			end
		end
		return self.killAverage
	end,
	
	GetAverageKillsRemaining = function(self)
		local xpRemaining, killsRemaining;

		xpRemaining = (self.maxXP or 0) - (self.xp or 0)
		killsRemaining = ceil(xpRemaining / self:GetAverageKillXP());
		
		return killsRemaining;
	end,
	
	ClearKillList = function (self, initialValue)
		sData.pet.killList = { }
        if initialValue ~= nil and tonumber(initialValue) > 0 then
            table.insert(sData.pet.killList, tonumber(initialValue))
        end
	end,
	
	GetMobName = function(self)
		if self.nextMobName then
			local name = self.nextMobName
			self.nextMobName = nil
			return name
		else
			return strlower(L["Kills"])
		end
	end,
	
	GetName = function(self)
		if not self.name then
			self.name = UnitName("pet")
		end
		return self.name -- /run cDisplay:Print(cPet:GetName())
	end,
    
    --
    -- Clear methods
    --
    ClearKills = function(self)
        sData.pet.killList = { }
        self.killAverage = nil;
    end,
};