﻿cPlayer = {
	-- Members
	isActive = false,
	
	level = nil,
	maxLevel = 80, -- Assume WotLK-enabled. Will be corrected once properly initialized.
	currentXP = nil,
    restedXP = 0,
	maxXP = nil,
    
    killAverage = nil,
	questAverage = nil,
	killList = { },
	questList = { },
	killListLength = 100, -- The max allowed value, not the current selection.
	questListLength = 100,
	bgListLength = 300,
	
	hasEnteredBG = true,
	bgAverage = nil,
	bgObjAverage = nil,
	
	-- Constructor
	Initialize = function(self)
		self:SyncData()
        
		if self.level == self.maxLevel then
			self.isActive = false
		else
			self.isActive = true
		end
		
        self.killAverage = nil
		self.bgObjAverage = nil
        self.questAverage = nil

        self.maxLevel = self:GetMaxLevel();
	end,

    -- Calculates the max level for the player, based on the expansion level
    -- available to the player.
    -- This assumes a 10 level increase per expansion, starting at level 60.
    GetMaxLevel = function(self)
        return (60 + (10 * GetAccountExpansionLevel()));
    end,
	
	CreateBgDataArray = function(self)
		return {
			inProgress = false,
			name = nil,
			totalXP = 0,
			objTotal = 0,
			objCount = 0,
			killCount = 0,
			killTotal = 0,
			objMinorTotal = 0,
			objMinorCount = 0,
		}
	end,
    
    ---
    -- Syncronizes the estimated data with the actual data.
    ---
    SyncData = function(self)
        self.level = UnitLevel("player")
		self.currentXP = UnitXP("player")
		self.maxXP = UnitXPMax("player")
        
        local rested = GetXPExhaustion() or 0
        self.restedXP = rested / 2
    end,
	
	-- Add methods
	AddKill = function(self, xpGained, mobName)
        self.currentXP = self.currentXP + xpGained
        
        local killXP = xpGained
        if self.restedXP > 0 then
            if self.restedXP > xpGained / 2 then
                killXP = xpGained / 2
                self.restedXP = self.restedXP - killXP
            else
                killXP = xpGained - self.restedXP
                self.restedXP = 0
            end
        end
    
		self.killAverage = nil
        table.insert(sData.player.killList, 1, {mob=mobName, xp=killXP})
		if(# sData.player.killList > self.killListLength) then
			table.remove(sData.player.killList)
		end

        return killXP
	end,
	
	AddQuest = function (self, xpGained)
		self.questAverage = nil
		self.currentXP = self.currentXP + xpGained
        table.insert(sData.player.questList, 1, xpGained)
		if(# sData.player.questList > self.questListLength) then
			table.remove(sData.player.questList)
		end
	end,
	
	-- Returns TRUE if a real objective is in question, FALSE if a minor objective is detected.
	AddBattlegroundObjective = function(self, xpGained)
		if sData.player.bgList[1].inProgress then
			if xpGained > GetBGObjectiveMinXP() then
				self.bgObjAverage = nil
				sData.player.bgList[1].totalXP = sData.player.bgList[1].totalXP + xpGained
				sData.player.bgList[1].objTotal = sData.player.bgList[1].objTotal + xpGained
				sData.player.bgList[1].objCount = sData.player.bgList[1].objCount + 1
				return true
			else
				--sData.player.bgList[1].objMinorTotal = sData.player.bgList[1].objMinorTotal + xpGained
				--sData.player.bgList[1].objMinorCount = sData.player.bgList[1].objMinorCount + 1
				return self:AddBattlegroundKill(xpGained, 'Unknown')
			end
		else
			cDisplay:Debug("Attempt to add a BG objective without starting a BG.")
		end
	end,
	
	AddBattlegroundKill = function(self, xpGained, name)
		if sData.player.bgList[1].inProgress then
			sData.player.bgList[1].totalXP = sData.player.bgList[1].totalXP + xpGained
			sData.player.bgList[1].killCount = sData.player.bgList[1].killCount + 1
			sData.player.bgList[1].killTotal = sData.player.bgList[1].killTotal + xpGained
		else
			cDisplay:Debug("Attempt to add a BG kill without starting a BG.")
		end
	end,
	
	BattlegroundStart = function(self, bgName)
		if (# sData.player.bgList) > 0 and sData.player.bgList[1].inProgress == true then
			cDisplay:Debug("Attempted to start a BG while another one is in progress.")
		else
			local bgDataArray = self:CreateBgDataArray();
			table.insert(sData.player.bgList, 1, bgDataArray)
			if(# sData.player.bgList > self.bgListLength) then
				table.remove(sData.player.bgList)
			end
			
			sData.player.bgList[1].inProgress = true
			sData.player.bgList[1].name = bgName or false
			cDisplay:Debug("BG Started! (" .. tostring(sData.player.bgList[1].name) .. ")")
		end
	end,
	
	BattlegroundEnd = function(self)
		if sData.player.bgList[1].inProgress == true then
			sData.player.bgList[1].inProgress = false
			cDisplay:Debug("BG Ended! (" .. tostring(sData.player.bgList[1].name)  .. ")")
			
			if sData.player.bgList[1].totalXP == 0 then
				table.remove(sData.player.bgList, 1)
				cDisplay:Debug("BG ended without any honor gain. Disregarding it.)")
			end
			
			self.bgAverage = nil
			self.bgObjAverage = nil
		else
			cDisplay:Debug("Attempted to end a BG before one was started.")
		end
	end,
	
	IsBattlegroundInProgress = function(self)
		return sData.player.bgList[1].inProgress
	end,
    
    ---
    -- Functions to calculate exact numbers of kills / quests needed
    -- based on the XP passed to them.
    ---
    GetKillsRequired = function(self, xp)
        if xp > 0 then
            local xpRemaining = self.maxXP - self.currentXP
            local xpRested = self:IsRested()
            if xpRested then
                if((xpRemaining / 2) > xpRested) then
                    xpRemaining = xpRemaining - xpRested
                else
                    xpRemaining = xpRemaining / 2
                end
            end
            return ceil(xpRemaining / xp)
        else
            return -1
        end
    end,
    
    GetQuestsRequired = function(self, xp)
        local xpRemaining = self.maxXP - self.currentXP
		if(xp > 0) then
			return ceil(xpRemaining / xp)
        else
            return -1
        end
    end,
    
    ---
    -- Average calculations
    -- Return estimates if there is no data to base the average values on.
    ---
	GetAverageKillXP = function (self)
		if self.killAverage == nil then
			if(# sData.player.killList > 0) then
				local total = 0
				local maxUsed = # sData.player.killList
				if maxUsed > sConfig.averageDisplay.playerKillListLength then
					maxUsed = sConfig.averageDisplay.playerKillListLength
				end
				for index, value in ipairs(sData.player.killList) do
					if index > maxUsed then
						break;
					end
					total = total + value.xp
				end
				self.killAverage = (total / maxUsed);
			else
				self.killAverage = MobXP()
			end
		end

        -- Recruit A Friend beta test.
        -- Simply tripples the DISPLAY value. The actual data remains intact.
        if IsRafApplied() then 
            return (self.killAverage * 3);
        else
            return self.killAverage
        end
	end,
    GetAverageKillsRemaining = function (self)
		if(self:GetAverageKillXP() > 0) then
            return self:GetKillsRequired(self:GetAverageKillXP())
		else
			return -1
		end
	end,
	
	GetAverageQuestXP = function (self)
		if self.questAverage == nil then
			if(# sData.player.questList > 0) then
				local total = 0
				local maxUsed = # sData.player.questList
				if maxUsed > sConfig.averageDisplay.playerQuestListLength then
					maxUsed = sConfig.averageDisplay.playerQuestListLength
				end
				for index, value in ipairs(sData.player.questList) do
					if index > maxUsed then
						break;
					end
					total = total + value
				end
				self.questAverage = (total / maxUsed);
			else
				-- A very VERY rought and quite possibly very wrong estimate.
				-- But it is accurate for the first few levels, which is where the inaccuracy would be most visible, so...
				self.questAverage = MobXP() * math.floor(((self.level + 9) / (self.maxLevel + 9)) * 20)
			end
		end
        -- Recruit A Friend beta test.
        -- Simply tripples the DISPLAY value. The actual data remains intact.
        if IsRafApplied() then 
            return (self.questAverage * 3);
        else
            return self.questAverage
        end
	end,
	GetAverageQuestsRemaining = function (self)
		local xpRemaining = self.maxXP - self.currentXP
		if(self:GetAverageQuestXP() > 0) then
			return self:GetQuestsRequired(self:GetAverageQuestXP())
		else
			return -1
		end
    end,
	
	---
    -- Battleground averages
    ---
	GetAverageBGXP = function (self)
		if self.bgAverage == nil then
			if(# sData.player.bgList > 0) then
				local total = 0
				local maxUsed = # sData.player.bgList
				if maxUsed > sConfig.averageDisplay.playerBGListLength then
					maxUsed = sConfig.averageDisplay.playerBGListLength
				end
				for index, value in ipairs(sData.player.bgList) do
					if index > maxUsed then
						break;
					end
					total = total + value.totalXP
				end
				self.bgAverage = (total / maxUsed)
			else
				self.bgAverage = (MobXP() * math.floor(((self.level + 9) / (self.maxLevel + 9)) * 20)) * 2
			end
		end
		return self.bgAverage
	end,
	GetAverageBGsRemaining = function(self)
		local bgAverage = self:GetAverageBGXP()
		if(bgAverage > 0) then
			local xpRemaining = self.maxXP - self.currentXP
			return ceil(xpRemaining / bgAverage)
		else
			return nil
		end
	end,
	
	GetAverageBGObjectiveXP = function (self)
		if self.bgObjAverage == nil then
			if(# sData.player.bgList > 0) then
				local total = 0
				local count = 0
				for index, value in ipairs(sData.player.bgList) do
					if value.objTotal > 0 and value.objCount > 0 then
						total = total + (value.objTotal / value.objCount)
						count = count + 1
					end
				end
				if count == 0 then
					self.bgObjAverage = 0
				else
					self.bgObjAverage = (total / count)
				end
			else
				self.bgObjAverage = self:GetAverageQuestXP()
			end
		end
		return self.bgObjAverage
	end,
	GetAverageBGObjectivesRemaining = function (self)
		local objAverage = self:GetAverageBGObjectiveXP()
		if(objAverage > 0) then
			local xpRemaining = self.maxXP - self.currentXP
			return ceil(xpRemaining / objAverage)
		else
			return nil	
		end
	end,
	
	GetBattlegroundsListed = function (self)
		if(# sData.player.bgList > 0) then
			local bgs = { }
			for index, value in ipairs(sData.player.bgList) do
				bgs[value.name] = (bgs[value.name] or 0) + 1
			end
			return bgs;
		else
			return nil
		end
	end,
	GetBattlegroundAverage = function(self, name)
		if(# sData.player.bgList > 0) then
			local total = 0
			local count = 0
			for index, value in ipairs(sData.player.bgList) do
				if value.name == name and not value.inProgress then
					total = total + value.totalXP
					count = count + 1
				end
			end
			if count == 0 then
				return 0
			else
				return round(total / count, 0)
			end
		else
			return 0
		end
	end,
	GetLatestBattlegroundDetails = function(self)
		if # sData.player.bgList > 0 then
			local data = { 
				totalXP = sData.player.bgList[1].totalXP,
				objCount = sData.player.bgList[1].objCount,
				killCount = sData.player.bgList[1].killCount,
				xpPerObj = 0,
				xpPerKill = 0,
				otherXP = sData.player.bgList[1].totalXP - (sData.player.bgList[1].objTotal + sData.player.bgList[1].killTotal),
			}
			
			if data.objCount > 0 then
				data.xpPerObj = round(sData.player.bgList[1].objTotal / data.objCount, 0)
			end
			if data.killCount > 0 then
				data.xpPerKill = round(sData.player.bgList[1].killTotal / data.killCount, 0)
			end
			
			return data
		else
			return nil
		end
	end,
	
	---
    -- Clear methods
    ---
	ClearKills = function (self, initialValue)
		sData.player.killList = { }
        self.killAverage = nil;
        if initialValue ~= nil and tonumber(initialValue) > 0 then
            table.insert(sData.player.killList, {mob='Initial', xp=tonumber(initialValue)})
        end
	end,
	ClearQuests = function (self, initialValue)
		sData.player.questList = { }
        self.questAverage = nil;
        if initialValue ~= nil and tonumber(initialValue) > 0 then
            table.insert(sData.player.questList, tonumber(initialValue))
		end
	end,
	ClearBattlegrounds = function(self, initialValue)
		sData.player.bgList = { }
        self.bgAverage = nil;
        self.bgObjAverage = nil;
        if initialValue ~= nil and tonumber(initialValue) > 0 then
            table.insert(sData.player.bgList, tonumber(initialValue))
		end
	end,
    
    ---
    -- Checks whether the player is rested.
    -- Retuns the additional XP the player will get until he is 
    -- unrested again, or FALSE if the player is not rested.
    IsRested = function(self)
        if self.restedXP > 0 then
            return self.restedXP
        else
            return false
        end
    end
};
